/*
 * A context resource offering support for opening files.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "opener_context_resource.h"
#include "opener_context_object_server.h"
#include "opener_resource.h"

#include <string.h>



/* Support for providing access to files. */

OpenerContextResource::OpenerContextResource(OpenerResource *opener, Memory *memory)
: SimplePager(memory), _opener(opener)
{
}

int OpenerContextResource::expected_items()
{
    return OpenerContextObject_expected_items;
}

ipc_server_handler_type OpenerContextResource::handler()
{
    return (ipc_server_handler_type) handle_OpenerContextObject;
}



/* Data access methods. */

char *OpenerContextResource::get_path()
{
    char *buffer = _region->read();
    offset_t size = _region->size();

    /* Confine the path to the limit of the buffer. */

    if ((buffer != NULL) && (strnlen(buffer, size) < size))
        return buffer;
    else
        return NULL;
}



/* Opener context interface methods. */

long OpenerContextResource::open(flags_t flags, offset_t *size, l4_cap_idx_t *file)
{
    char *path = get_path();

    if (path == NULL)
        return -L4_EINVAL;

    return _opener->open(path, flags, size, file);
}

// vim: tabstop=4 expandtab shiftwidth=4
